/// Gemini protocol implementation
library gemini_protocol;

import 'dart:convert';

/// Client to server request
class Request {
  /// Requested URL
  final Uri url;

  const Request({required this.url});

  /// Construct a request from byte stream (e.g. TCP socket)
  static Future<Request> fromByteStream(Stream<List<int>> stream) async {
    final url = Uri.parse(await stream
        .transform(const Utf8Decoder())
        .transform(const LineSplitter())
        .first);

    return Request(url: url);
  }

  /// Encode the request to raw byte array
  List<int> encode() => utf8.encode("$url\r\n");
}

/// Server to client response
class Response {
  /// Status code
  final int statusCode;

  /// Metadata
  final String meta;

  /// Raw body bytes
  final List<int> body;

  const Response({
    required this.statusCode,
    required this.meta,
    required this.body,
  });

  /// Construct a response from byte stream (e.g. TCP socket)
  static Future<Response> fromByteStream(Stream<List<int>> data) async {
    final parts = await data
        .transform(const Utf8Decoder())
        .transform(const LineSplitter())
        .first;

    final statusCode = int.parse(parts.substring(0, 2));
    final meta = parts.substring(3);
    final body = (await data.expand((element) => element).toList())
        // [parts] transformer does not consume [data]
        // for some reason so let's use this dirty hack
        .sublist(parts.length + 2);

    return Response(statusCode: statusCode, meta: meta, body: body);
  }

  /// Encode the request to raw byte array
  List<int> encode() => utf8.encode("$statusCode $meta\r\n") + body;
}
